@@ -5,7 +5,7 @@ This is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2021, Free Software Foundation, Inc.
+Copyright (C) 2015-2022 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -220,11 +220,8 @@ gal_wcs_read_fitsptr(fitsfile *fptr, int linearmatrix, size_t hstartwcs,
   status=wcspih(fullheader, nkeys, relax, ctrl, &nreject, nwcs, &wcs);
   if(status)
     {
-      fprintf(stderr,"\n"
-              "##################\n"
-              "WCSLIB Warning: wcspih ERROR %d: %s.\n"
-              "##################\n",
-              status, wcs_errmsg[status]);
+      error(EXIT_SUCCESS, 0, "%s: WCSLIB Warning: wcspih ERROR %d: %s",
+            __func__, status, wcs_errmsg[status]);
       wcs=NULL; *nwcs=0;
     }
 
@@ -358,10 +355,8 @@ gal_wcs_read_fitsptr(fitsfile *fptr, int linearmatrix, size_t hstartwcs,
           status=wcsset(wcs);
           if(status)
             {
-              fprintf(stderr, "\n##################\n"
-                      "WCSLIB Warning: wcsset ERROR %d: %s.\n"
-                      "##################\n",
-                      status, wcs_errmsg[status]);
+              error(EXIT_SUCCESS, 0, "%s: WCSLIB warning: wcsset "
+                    "error %d: %s", __func__, status, wcs_errmsg[status]);
               wcsfree(wcs);
               wcs=NULL;
               *nwcs=0;
@@ -473,8 +468,8 @@ gal_wcs_create(double *crpix, double *crval, double *cdelt,
   /* Set up the wcs structure with the constants defined above. */
   status=wcsset(wcs);
   if(status)
-    error(EXIT_FAILURE, 0, "wcsset error %d: %s", status,
-          wcs_errmsg[status]);
+    error(EXIT_FAILURE, 0, "%s: wcsset error %d: %s", __func__,
+          status, wcs_errmsg[status]);
 
   /* If a CD matrix is desired make it. */
   if(linearmatrix==GAL_WCS_LINEAR_MATRIX_CD)
@@ -1620,6 +1615,11 @@ gal_wcs_warp_matrix(struct wcsprm *wcs)
       out[1] = -1 * wcs->cdelt[1] *sin(crota2);
       out[2] = wcs->cdelt[0] * sin(crota2);
       out[3] = wcs->cdelt[1] * cos(crota2);
+
+      /* For a check:
+      printf("cdelt: %f, %f\n", wcs->cdelt[0], wcs->cdelt[1]);
+      printf("cd:\n%f, %f\n%f, %f\n", out[0], out[1], out[2], out[3]);
+      */
     }
   else
     error(EXIT_FAILURE, 0, "%s: currently only PCi_ja and CDi_ja keywords "
@@ -1676,8 +1676,7 @@ gal_wcs_clean_errors(struct wcsprm *wcs)
 /* According to the FITS standard, in the 'PCi_j' WCS formalism, the matrix
    elements m_{ij} are encoded in the 'PCi_j' keywords and the scale
    factors are encoded in the 'CDELTi' keywords. There is also another
-   formalism (the 'CDi_j' formalism) which merges the two into one
-   matrix.
+   formalism (the 'CDi_j' formalism) which merges the two into one matrix.
 
    However, WCSLIB's internal operations are apparently done in the 'PCi_j'
    formalism. So its outputs are also all in that format by default. When
@@ -1689,6 +1688,7 @@ void
 gal_wcs_decompose_pc_cdelt(struct wcsprm *wcs)
 {
   size_t i, j;
+  int status=0;
   double *ps, *warp;
 
   /* If there is on WCS, then don't do anything. */
@@ -1700,7 +1700,7 @@ gal_wcs_decompose_pc_cdelt(struct wcsprm *wcs)
   if(ps==NULL) return;
 
   /* For a check.
-  printf("pc: %g, %g\n", pc[0], pc[1]);
+  printf("ps: %g, %g\n", ps[0], ps[1]);
   printf("warp: %g, %g, %g, %g\n", warp[0], warp[1],
          warp[2], warp[3]);
   */
@@ -1723,6 +1723,12 @@ gal_wcs_decompose_pc_cdelt(struct wcsprm *wcs)
      assumptioins about 'CDELTi'. */
   wcs->altlin=1;
 
+  /* Re-run 'wcsset' to update all the internal WCS parameters. */
+  status=wcsset(wcs);
+  if(status)
+    error(EXIT_SUCCESS, 0, "%s: WCSLIB warning: wcsset ERROR %d: %s",
+          __func__, status, wcs_errmsg[status]);
+
   /* Clean up. */
   free(ps);
   free(warp);
@@ -2245,8 +2251,8 @@ wcs_convert_sanity_check_alloc(gal_data_t *coords, struct wcsprm *wcs,
    list. But in WCSLIB, the input is a single array (with multiple
    columns). This function will convert between the two. */
 static void
-wcs_convert_list_to_array(gal_data_t *list, double *array, int *stat,
-                          size_t ndim, int listtoarray)
+wcs_convert_list_to_from_array(gal_data_t *list, double *array, int *stat,
+                               size_t ndim, int to0from1)
 {
   size_t i, d=0;
   gal_data_t *tmp;
@@ -2257,10 +2263,10 @@ wcs_convert_list_to_array(gal_data_t *list, double *array, int *stat,
          input into or output from WCSLIB. */
       for(i=0;i<list->size;++i)
         {
-          if(listtoarray)
-            array[i*ndim+d] = ((double *)(tmp->array))[i];
-          else
+          if(to0from1)
             ((double *)(tmp->array))[i] = stat[i] ? NAN : array[i*ndim+d];
+          else
+            array[i*ndim+d] = ((double *)(tmp->array))[i];
         }
 
       /* Increment the dimension. */
@@ -2313,7 +2319,7 @@ gal_wcs_world_to_img(gal_data_t *coords, struct wcsprm *wcs, int inplace)
 
   /* Write the values from the input list of separate columns into a single
      array (WCSLIB input). */
-  wcs_convert_list_to_array(coords, world, stat, wcs->naxis, 1);
+  wcs_convert_list_to_from_array(coords, world, stat, wcs->naxis, 0);
 
 
   /* Use WCSLIB's wcss2p for the conversion. */
@@ -2341,7 +2347,7 @@ gal_wcs_world_to_img(gal_data_t *coords, struct wcsprm *wcs, int inplace)
 
   /* Write the output from a single array (WCSLIB output) into the output
      list of this function. */
-  wcs_convert_list_to_array(out, pixcrd, stat, wcs->naxis, 0);
+  wcs_convert_list_to_from_array(out, pixcrd, stat, wcs->naxis, 1);
 
 
   /* Clean up. */
@@ -2349,6 +2355,7 @@ gal_wcs_world_to_img(gal_data_t *coords, struct wcsprm *wcs, int inplace)
   free(stat);
   free(theta);
   free(world);
+  free(imgcrd);
   free(pixcrd);
 
   /* Return the output list of coordinates. */
@@ -2375,7 +2382,7 @@ gal_wcs_img_to_world(gal_data_t *coords, struct wcsprm *wcs, int inplace)
 
   /* Write the values from the input list of separate columns into a single
      array (WCSLIB input). */
-  wcs_convert_list_to_array(coords, pixcrd, stat, wcs->naxis, 1);
+  wcs_convert_list_to_from_array(coords, pixcrd, stat, wcs->naxis, 0);
 
 
   /* Use WCSLIB's wcsp2s for the conversion. */
@@ -2413,7 +2420,7 @@ gal_wcs_img_to_world(gal_data_t *coords, struct wcsprm *wcs, int inplace)
 
   /* Write the output from a single array (WCSLIB output) into the output
      list of this function. */
-  wcs_convert_list_to_array(out, world, stat, wcs->naxis, 0);
+  wcs_convert_list_to_from_array(out, world, stat, wcs->naxis, 1);
 
 
   /* Clean up. */
@@ -2421,6 +2428,7 @@ gal_wcs_img_to_world(gal_data_t *coords, struct wcsprm *wcs, int inplace)
   free(stat);
   free(theta);
   free(world);
+  free(imgcrd);
   free(pixcrd);
 
 
